@@ -1,6 +1,6 @@
 import cerberus
 from pathlib import Path
-from typing import Union
+from typing import Union,List
 
 
 class UniqueNameChecker:
@@ -17,6 +17,16 @@ class UniqueNameChecker:
             self.name_lookup.append(value)
 
 
+class DefinesPrefixChecker:
+    def check(self, field, defines: List[str], error):
+        for value in defines:
+            if value.startswith("-D"):
+                error(
+                    field,
+                    f"Unnecessary -D prefix in {field}: {defines}. Aim will add this automatically.",
+                )
+
+
 class RequiresExistChecker:
     def __init__(self, document):
         self.doc = document
@@ -31,20 +41,18 @@ class RequiresExistChecker:
                 error(field, f"{value} does not match any build name. Check spelling.")
 
 
-class DirectoriesExistsChecker:
+class RelProjectDirPathChecker:
     def __init__(self, project_dir):
         self.project_dir = project_dir
 
-    def check(self, field, dirs, error):
-        dirs = [(self.project_dir / directory).resolve() for directory in dirs]
+    def check(self, field, paths, error):
+        paths = [(self.project_dir / the_path).resolve() for the_path in paths]
 
-        for directory in dirs:
+        for directory in paths:
+            # Remember paths can now be directories or specific paths to files.
             if not directory.exists():
                 error(field, f"{str(directory)} does not exist.")
                 break
-            if not directory.is_dir():
-                error(field, f"{str(directory)} is not a dir.")
-                break
 
 
 class AimCustomValidator(cerberus.Validator):
@@ -56,17 +64,17 @@ class AimCustomValidator(cerberus.Validator):
         # TODO: check outputNames are unique to prevent dependency cycle.
 
         def check_convention(_field, _value):
-            errors = []
+            the_errors = []
             if _value.startswith("lib"):
-                error_str = "You should not prefix names with 'lib'."
-                errors.append(error_str)
+                the_error_str = "You should not prefix names with 'lib'."
+                the_errors.append(the_error_str)
 
             suffix = Path(_value).suffix
             if suffix:
-                error_str = f"You should not specify the suffix."
-                errors.append(error_str)
+                the_error_str = f"You should not specify the suffix."
+                the_errors.append(the_error_str)
 
-            return errors
+            return the_errors
 
         # Bit of a hack so strings go through the same code path as lists.
         if isinstance(value, str):
@@ -89,19 +97,22 @@ class AimCustomValidator(cerberus.Validator):
 def target_schema(document, project_dir):
     unique_name_checker = UniqueNameChecker()
     requires_exist_checker = RequiresExistChecker(document)
-    dir_exists_checker = DirectoriesExistsChecker(project_dir)
+    path_checker = RelProjectDirPathChecker(project_dir)
+    defines_checker = DefinesPrefixChecker()
 
     schema = {
-        "cxx": {"required": True, "type": "string"},
-        "cc": {"required": True, "type": "string"},
+        "compiler": {"required": True, "type": "string"},
         "ar": {"required": True, "type": "string"},
         "compilerFrontend": {
             "required": True,
             "type": "string",
             "allowed": ["msvc", "gcc", "osx"],
         },
-        "flags": {"type": "list", "schema": {"type": "string"}},
-        "defines": {"type": "list", "schema": {"type": "string"}},
+        "flags": {"type": "list", "schema": {"type": "string"}, "empty": False},
+        "defines": {"type": "list",
+                    "schema": {"type": "string"},
+                    "empty": False,
+                    "check_with": defines_checker.check},
         "projectRoot": {"required": True, "type": "string", "empty": False},
         "builds": {
             "required": True,
@@ -114,6 +125,18 @@ def target_schema(document, project_dir):
                         "type": "string",
                         "check_with": unique_name_checker.check,
                     },
+                    "compiler": {"required": False, "type": "string"},
+                    "defines": {
+                        "type": "list",
+                        "schema": {"type": "string"},
+                        "empty": False,
+                        "check_with": defines_checker.check,
+                    },
+                    "flags": {
+                        "type": "list",
+                        "schema": {"type": "string"},
+                        "empty": False,
+                    },
                     "requires": {
                         "type": "list",
                         "empty": False,
@@ -135,19 +158,20 @@ def target_schema(document, project_dir):
                         "empty": False,
                         "type": "list",
                         "schema": {"type": "string"},
-                        "check_with": dir_exists_checker.check,
+                        "check_with": path_checker.check,
                     },
                     "includePaths": {
                         "type": "list",
                         "empty": False,
                         "schema": {"type": "string"},
-                        "check_with": dir_exists_checker.check,
+                        "check_with": path_checker.check,
                     },
                     "libraryPaths": {
                         "type": "list",
                         "empty": False,
                         "schema": {"type": "string"},
-                        "check_with": dir_exists_checker.check,
+                        # you can't check the library dirs as they may not exist if the project not built before.
+                        # "check_with": path_checker.check,
                         "dependencies": {"buildRule": ["exe", "dynamiclib"]},
                     },
                     "libraries": {
